@@ -69,6 +69,7 @@ from kestrel.syntax.utils import (
     all_relations,
     LITERALS,
     AGG_FUNCS,
+    TRANSFORMS,
 )
 from kestrel.semantics import *
 from kestrel.codegen import commands
@@ -225,12 +226,14 @@ class Session(AbstractContextManager):
                 runtime_directory_master.unlink()
             runtime_directory_master.symlink_to(self.runtime_directory)
 
-        # local database of SQLite or Parquet
+        # local database of SQLite or PostgreSQL
         if not store_path:
             # use the default local database in config.py
-            store_path = os.path.join(
-                self.runtime_directory, self.config["session"]["local_database_path"]
-            )
+            local_database_path = self.config["session"]["local_database_path"]
+            if "://" in local_database_path:
+                store_path = local_database_path
+            else:
+                store_path = os.path.join(self.runtime_directory, local_database_path)
         self.store = get_storage(store_path, self.session_id)
 
         # Symbol Table
@@ -425,6 +428,8 @@ class Session(AbstractContextManager):
                             tmp.extend(all_relations)
                     elif token == "FUNCNAME":
                         tmp.extend(AGG_FUNCS)
+                    elif token == "TRANSFORM":
+                        tmp.extend(TRANSFORMS)
                     elif token in LITERALS:
                         continue
                     elif token.startswith("__ANON"):
@@ -492,7 +497,7 @@ class Session(AbstractContextManager):
 
             # exception completion
             except StixPatternError as e:
-                raise InvalidStixPattern(e.stix)
+                raise InvalidStixPattern(e.stix) from e
 
             # post-processing: symbol table update
             if output_var_struct is not None:
