@@ -1,7 +1,9 @@
 import ast
-from lark import Lark, Transformer, Tree
 from pkgutil import get_data
 
+from firepit.query import Filter, Predicate
+from lark import Lark, Transformer, Tree
+
 
 def parse(stmts, default_variable="_", default_sort_order="desc"):
     # the public parsing interface for Kestrel
@@ -41,6 +43,11 @@ class _PostParsing(Transformer):
         stmt["output"] = _extract_var(args, self.default_variable)
         return stmt
 
+    def assign(self, args):
+        result = args[0]  # Already transformed in expression method below
+        result["command"] = "assign"
+        return result
+
     def merge(self, args):
         return {
             "command": "merge",
@@ -51,13 +58,13 @@ class _PostParsing(Transformer):
         return {"command": "info", "input": _extract_var(args, self.default_variable)}
 
     def disp(self, args):
-        paths = _assert_and_extract_single("STIXPATHS", args)
-        return {
-            "command": "disp",
-            "input": _extract_var(args, self.default_variable),
-            "attrs": paths if paths else "*",
-            "limit": _extract_int(args),
-        }
+        result = {"command": "disp"}
+        for arg in args:
+            if isinstance(arg, dict):
+                result.update(arg)
+        if "attrs" not in result:
+            result["attrs"] = "*"
+        return result
 
     def get(self, args):
         datasource = _extract_datasource(args)
@@ -169,6 +176,45 @@ class _PostParsing(Transformer):
             "data": _assert_and_extract_single("VARDATA", args),
         }
 
+    def expression(self, args):
+        result = args[0]
+        for arg in args:
+            result.update(arg)
+        return result
+
+    def transform(self, args):
+        return {
+            "input": _extract_var(args, self.default_variable),
+            "transform": _assert_and_extract_single("TRANSFORM", args),
+        }
+
+    def where_clause(self, args):
+        return {
+            "where": Filter([args[0]]),
+        }
+
+    def attr_clause(self, args):
+        paths = _assert_and_extract_single("STIXPATHS", args)
+        return {
+            "attrs": paths if paths else "*",
+        }
+
+    def sort_clause(self, args):
+        return {
+            "path": _extract_stixpath(args),
+            "ascending": _extract_direction(args, self.default_sort_order),
+        }
+
+    def limit_clause(self, args):
+        return {
+            "limit": _extract_int(args),
+        }
+
+    def offset_clause(self, args):
+        return {
+            "offset": _extract_int(args),
+        }
+
     def starttime(self, args):
         return {"start": _assert_and_extract_single("ISOTIMESTAMP", args)}
 
@@ -202,6 +248,44 @@ class _PostParsing(Transformer):
         alias = args[2].value if len(args) > 2 else f"{func}_{args[1].value}"
         return {"func": func, "attr": args[1].value, "alias": alias}
 
+    def disj(self, args):
+        lhs = str(args[0])
+        rhs = str(args[2])
+        return Predicate(lhs, "OR", rhs)
+
+    def conj(self, args):
+        lhs = str(args[0])
+        rhs = str(args[2])
+        return Predicate(lhs, "AND", rhs)
+
+    def comp(self, args):
+        lhs = str(args[0])
+        op = str(args[1])
+        rhs = str(args[2])
+        return Predicate(lhs, op, rhs)
+
+    def null_comp(self, args):
+        lhs = str(args[0])
+        op = str(args[1])
+        if op == "IS NOT":
+            op = "!="
+        else:
+            op = "="
+        rhs = "NULL"
+        return Predicate(lhs, op, rhs)
+
+    def column(self, args):
+        return args[0].value
+
+    def squoted_str(self, args):
+        return args[0].value.strip("'")
+
+    def num_literal(self, args):
+        return args[0].value
+
+    def null(self, args):
+        return "NULL"
+
 
 def _first(args):
     return args[0].value
